@@ -26,16 +26,19 @@ def louvain_community_step(
     pass  # pragma: no cover
 
 
-@abstract_algorithm("util.graph.collapse_by_label")
-def collapse_by_label(
-    graph: Graph(is_directed=False),
-    labels: NodeMap,
-    aggregator: Callable[[Any, Any], Any],
-) -> Graph:
-    pass  # pragma: no cover
-
-
+# TODO: why is this "cluster" instead of "clustering"?
 @abstract_algorithm("cluster.triangle_count")
 def triangle_count(graph: Graph(is_directed=False)) -> int:
     """Counts the number of unique triangles in an undirected graph"""
     pass  # pragma: no cover
+
+
+@abstract_algorithm("clustering.coloring.greedy")
+def greedy_coloring(graph: Graph(is_directed=False)) -> Tuple[NodeMap, int]:
+    """
+    Attempts to find the minimum number of colors required to color the graph such that no connected
+    nodes have the same color. Color is simply represented as a value from 0..n
+
+    Returns color for each node and # of colors required
+    """
+    pass  # pragma: no covert
